@@ -107,7 +107,7 @@ static struct cnfparamblk modpblk =
 static prop_t *pInputName = NULL;	/* there is only one global inputName for all messages generated by this module */
 static prop_t *pLocalHostIP = NULL;
 
-static inline void
+static void
 initConfigSettings(void)
 {
 	cs.bPermitNonKernel = 0;
