@@ -321,7 +321,7 @@ createInstance(instanceConf_t **pinst)
 	inst->bWritePid = 0;
 	inst->bAnnotate = 0;
 	inst->bParseTrusted = 0;
-	inst->bDiscardOwnMsgs = 1;
+	inst->bDiscardOwnMsgs = bProcessInternalMessages;
 	inst->bUnlink = 1;
 	inst->next = NULL;
 
@@ -479,7 +479,7 @@ static rsRetVal discardLogSockets(void)
 
 /* used to create a log socket if NOT passed in via systemd. 
  */
-static inline rsRetVal
+static rsRetVal
 createLogSocket(lstn_t *pLstn)
 {
 	struct sockaddr_un sunx;
@@ -508,7 +508,7 @@ finalize_it:
 }
 
 
-static inline rsRetVal
+static rsRetVal
 openLogSocket(lstn_t *pLstn)
 {
 	DEFiRet;
@@ -582,7 +582,7 @@ finalize_it:
  * Returns NULL if not found or rate-limiting not activated for this
  * listener (the latter being a performance enhancement).
  */
-static inline rsRetVal
+static rsRetVal
 findRatelimiter(lstn_t *pLstn, struct ucred *cred, ratelimit_t **prl)
 {
 	ratelimit_t *rl = NULL;
@@ -637,7 +637,7 @@ finalize_it:
 
 /* patch correct pid into tag. bufTAG MUST be CONF_TAG_MAXSIZE long!
  */
-static inline void
+static void
 fixPID(uchar *bufTAG, int *lenTag, struct ucred *cred)
 {
 	int i;
@@ -740,7 +740,7 @@ finalize_it:
  * It is assumed the output buffer is large enough. Returns the number of
  * characters added.
  */
-static inline int
+static int
 copyescaped(uchar *dstbuf, uchar *inbuf, int inlen)
 {
 	int iDst, iSrc;
@@ -761,7 +761,7 @@ copyescaped(uchar *dstbuf, uchar *inbuf, int inlen)
  * We now parse the message according to expected format so that we
  * can also mangle it if necessary.
  */
-static inline rsRetVal
+static rsRetVal
 SubmitMsg(uchar *pRcv, int lenRcv, lstn_t *pLstn, struct ucred *cred, struct timeval *ts)
 {
 	msg_t *pMsg = NULL;
@@ -1178,7 +1178,10 @@ CODESTARTbeginCnfLoad
 	pModConf->bParseTrusted = 0;
 	pModConf->bParseHost = UNSET;
 	pModConf->bUseSpecialParser = 1;
-	pModConf->bDiscardOwnMsgs = 1;
+	/* if we do not process internal messages, we will see messages
+	 * from ourselves, and so we need to permit this.
+	 */
+	pModConf->bDiscardOwnMsgs = bProcessInternalMessages;
 	pModConf->bUnlink = 1;
 	pModConf->ratelimitIntervalSysSock = DFLT_ratelimitInterval;
 	pModConf->ratelimitBurstSysSock = DFLT_ratelimitBurst;
