@@ -134,6 +134,9 @@ case $1 in
 		curl localhost:9200/rsyslog_testbench/_search?size=$2 > work
 		python $srcdir/es_response_get_msgnum.py > rsyslog.out.log
 		;;
+   'getpid')
+		pid=$(cat rsyslog$2.pid)
+		;;
    'startup')   # start rsyslogd with default params. $2 is the config file name to use
    		# returns only after successful startup, $3 is the instance (blank or 2!)
 		if [ "x$2" == "x" ]; then
@@ -145,7 +148,7 @@ case $1 in
 		    echo "ERROR: config file '$CONF_FILE' not found!"
 		    exit 1
 		fi
-		$valgrind ../tools/rsyslogd -C -n -irsyslog$3.pid -M../runtime/.libs:../.libs -f$CONF_FILE &
+		LD_PRELOAD=$RSYSLOG_PRELOAD $valgrind ../tools/rsyslogd -C -n -irsyslog$3.pid -M../runtime/.libs:../.libs -f$CONF_FILE &
 		. $srcdir/diag.sh wait-startup $3
 		;;
    'startup-silent')   # start rsyslogd with default params. $2 is the config file name to use
@@ -353,6 +356,25 @@ case $1 in
 		rm -f work
 		$RS_SORTCMD -g < rsyslog.out.log > work
 		;;
+   'assert-equal')
+		if [ "x$4" == "x" ]; then
+			tolerance=0
+		else
+			tolerance=$4
+		fi
+		result=$(echo $2 $3 $tolerance | awk 'function abs(v) { return v > 0 ? v : -v } { print (abs($1 - $2) <= $3) ? "PASSED" : "FAILED" }')
+		if [ $result != 'PASSED' ]; then
+				echo "Value '$2' != '$3' (within tolerance of $tolerance)"
+		  . $srcdir/diag.sh error-exit 1
+		fi
+		;;
+   'ensure-no-process-exists')
+    ps -ef | grep -v grep | grep -qF "$2"
+    if [ "x$?" == "x0" ]; then
+      echo "assertion failed: process with name-fragment matching '$2' found"
+		  . $srcdir/diag.sh error-exit 1
+    fi
+    ;;
    'seq-check') # do the usual sequence check to see if everything was properly received. $2 is the instance.
 		rm -f work
 		cp rsyslog.out.log work-presort
